import dolfyn.adv.api as avm
from dolfyn.tools import within
from dolfyn.data.time import date2num, num2date
import datetime
import numpy as np

"""
This data is from measurements at Admiralty Head, in Admiralty Inlet
(Puget Sound) in June of 2014. The measurements were made using
Inertial Motion Unit (IMU) equipped ADVs mounted on Tidal Turbulence
Mooring's (TTMs). The TTM positions the ADV head above the seafloor
to make mid-depth turbulence measurements. The inertial measurements
from the IMU allows for removal of mooring motion in post processing.

The mooring motion has been removed from the stream-wise and vertical
velocity signals (u, w). The lateral (v) velocity has some
'persistent motion contamination' due to mooring sway.

Each ttm was deployed with two ADVs. The 'top' ADV head was positioned 0.5m
above the 'bottom' ADV head. The TTMs were placed in 58m of water. The position of the TTMs were:

ttm01  : (48.1525, -122.6867)
ttm01b : (48.15256666, -122.68678333)
ttm02b : (48.152783333, -122.686316666)


Files
-----

This data contains the following files:

**<basefile>_<date>_Average5min.csv**

This file contains 5minute averages of the velocity, and the
turbulence intensity.

**<basename>_<date>.vec**

This is the full binary Nortek-format data file. For information on
this data format see Nortek's 'System Integration Manual' available
at: http://www.nortek-as.com/en/support/application-development

**<basename>_<date>_mc.h5**

This file contains a 'lightly processed' version of the raw data in
hdf5 format. This file can be read directly by the DOLfYN library's
'load' function, or using other hdf5 I/O tools such as HDFView. Time
is available in MatPlotLib time (mpltime) format.

**<basename>_<date>_mc.mat**

This file contains the key variables (u, urot, uacc, orientmat, Accel,
AngRt) from the .h5 file, in Matlab format. Time is available in
Matlab 'datenum' format.

**process_data.py**

A script that utilizes the DOLfYN library to read the binary .vec
file, and produce the other files in this dataset.


Units
-----

- Velocity data (_u, urot, uacc) is in m/s.
- Acceleration (Accel) data is in m/s^2.
- Angular rate (AngRt) data is in rad/s.
- The components of all vectors are in 'ENU' orientation. That is, the
  first index is True East, the second is True North, and the third is
  Up (vertical).
- All other quantities are in the units defined in the Nortek Manual.

Motion correction and rotation into the ENU earth reference frame was
performed using the Python-based open source DOLfYN library
(http://lkilcher.github.io/dolfyn/). Details on motion correction can
be found there.

Additional details on TTM measurements at this site can be found in
Thomson et al. 2013, 'Tidal Turbulence measurements from a compliant
mooring' Marine Energy Technology Symposium paper, available here:
http://faculty.washington.edu/jmt3rd/Publications/Thomson_etal_GMREC-METS_2013_turbulencemooring.pdf

"""


# The file names:
fnames = ['ttm01_ADVbottom_NREL01_June2014',
          'ttm01_ADVtop_NREL02_June2014',
          'ttm01b_ADVbottom_NREL01_June2014',
          'ttm01b_ADVtop_NREL02_June2014',
          'ttm02b_ADVbottom_F01_June2014',
          'ttm02b_ADVtop_NREL03_June2014']

# The time ranges that each TTM was on the seafloor:
# These are calculated by inspection of the signal (e.g. u, v, w, Accel,
# etc...) timeseries.
time_range = {'ttm01': [date2num(datetime.datetime(2014, 6, 16, 21, 12, 0)),
                        date2num(datetime.datetime(2014, 6, 17, 14, 42, 0)), ],
              'ttm02b': [date2num(datetime.datetime(2014, 6, 18, 8, 15, 0)),
                         date2num(datetime.datetime(2014, 6, 19, 5, 0, 0)), ],
              'ttm01b': [date2num(datetime.datetime(2014, 6, 18, 8, 0, 0)),
                         date2num(datetime.datetime(2014, 6, 19, 5, 12, 0)), ],
              }

# The lat/lon values
latlons = {'ttm01b': (48.15256666, -122.68678333),
           'ttm02b': (48.152783333, -122.686316666),
           'ttm01': (48.1525, -122.6867),
           }

# The body-head vectors:
m_in = 0.0254
b2h_vec = {'ttm01-ADVbottom': np.array([-11.5, -0.25, -14.25]) * m_in,
           'ttm01-ADVtop': np.array([0.254, -0.064, -0.165])
           }
b2h_vec['ttm01b-ADVtop'] = b2h_vec['ttm01-ADVtop']
b2h_vec['ttm01b-ADVbottom'] = b2h_vec['ttm01-ADVbottom']
b2h_vec['ttm02b-ADVtop'] = np.array([10., -2.5, -5]) * m_in  # in->m
b2h_vec['ttm02b-ADVbottom'] = np.array([-13.25, 2.75, -12.75]) * m_in  # in->m

# The rotation matrices
b2h_rotmat = {}
# They are all the same for the TTMs
(b2h_rotmat['ttm01b-ADVbottom'],
 b2h_rotmat['ttm01-ADVtop'],
 b2h_rotmat['ttm01-ADVbottom'],
 b2h_rotmat['ttm02b-ADVtop'],
 b2h_rotmat['ttm02b-ADVbottom'],
 b2h_rotmat['ttm01b-ADVtop'],) = 6 * [np.array([[0, 0, -1],
                                                [0, -1, 0],
                                                [-1, 0, 0]])]

binner = avm.TurbBinner(n_bin=10240, fs=32., n_fft=10240, n_fft_coh=2048)

mc = avm.motion.CorrectMotion()

for fnm in fnames:
    # Read the raw vector file
    dr = avm.read_nortek(fnm + '.vec')
    tag = '{}-{}'.format(*fnm.split('_')[:2])
    bnr = avm.TurbBinner(n_bin=5 * 60 * dr.fs, fs=dr.fs)

    # Assign some properties:
    dr.props['body2head_rotmat'] = b2h_rotmat[tag]
    if tag in b2h_vec.keys():
        dr.props['body2head_vec'] = b2h_vec[tag]
    dr.noise[0] = 0
    dr.noise[1] = 0
    dr.noise[2] = 0
    dr.props['latlon'] = latlons[tag.split('-')[0]]

    # Crop the data when the instrument was on the seafloor
    dr = dr.subset(within(dr.mpltime, time_range[tag.split('-')[0]]))

    ##########
    print('Cleaning the data...')
    dr.u[~within(dr.u, [-2.5, 0.5])] = np.NaN
    avm.clean.fillpoly(dr.u, 3, 12)
    dr.v[~within(dr.v, [-1, 1])] = np.NaN
    avm.clean.fillpoly(dr.v, 3, 12)
    dr.v[~within(dr.w, [-1, 1])] = np.NaN
    avm.clean.fillpoly(dr.w, 3, 12)
    avm.clean.GN2002(dr.u)
    avm.clean.GN2002(dr.v)
    avm.clean.GN2002(dr.w)
    for nm, d in dr:
        if isinstance(d, np.ndarray) and d.dtype == np.float64 and nm not in ['mpltime']:
            dr[nm] = d.astype(np.float32)
    if dr.has_imu():
        dr.pitch, dr.roll, dr.heading = avm.rotate.orient2euler(dr.orientmat)
    print('Saving...')
    dr.save(fnm + '_raw.h5',
            units={'_u': 'm/s', 'urot': 'm/s', 'uacc': 'm/s', 'AngRt': 'rad/s',
                   'Accel': 'm/s^2', 'AccelStable': 'm/s^2',
                   'pitch': 'deg', 'roll': 'deg', 'heading': 'deg true',
                   'mpltime': 'MatPlotLib Time format',
                   'time': 'ISO8601 time strings.', },
            description={'_u': 'Velocity array 0:True East, 1: True North, 2: Up',
                         'urot': 'The rotation-rate velocity.',
                         'uacc': 'The tranlational (acceleration) velocity.',
                         'AccelStable': 'The low-frequency acceleration '
                                        'that is ignored in calculating uacc.',
                         'pitch': 'The pitch of the ADV body',
                         'roll': 'The roll of the ADV body',
                         'heading': 'The heading (True) of the ADV body',
                         'orientmat': "The orientation matrix of the"
                                      " ADV body in the Earth's reference frame", })

    if dr.has_imu():
        print('Motion correcting...')
        drm = dr.copy()
        mc(drm)
        drm.pitch[:], drm.roll[:], drm.heading[:] = avm.rotate.orient2euler(drm.orientmat)
        print('Saving...')
        drm.save(fnm + '_mc.h5',
                 units={'_u': 'm/s', 'urot': 'm/s', 'uacc': 'm/s', 'AngRt': 'rad/s',
                        'Accel': 'm/s^2', 'AccelStable': 'm/s^2',
                        'pitch': 'deg', 'roll': 'deg', 'heading': 'deg true',
                        'mpltime': 'MatPlotLib Time format',
                        'time': 'ISO8601 time strings.', },
                 description={'_u': 'Velocity array 0:True East, 1: True North, 2: Up',
                              'urot': 'The rotation-rate velocity.',
                              'uacc': 'The tranlational (acceleration) velocity.',
                              'AccelStable': 'The low-frequency acceleration that'
                                             ' is ignored in calculating uacc.',
                              'pitch': 'The pitch of the ADV body',
                              'roll': 'The roll of the ADV body',
                              'heading': 'The heading (True) of the ADV body',
                              'orientmat': "The orientation matrix of the ADV "
                                           "body in the Earth's reference frame", }
        )

        print('Saving matlab file...')
        drm.add_data('datenum', drm.mpltime + 366, 'main')
        drm.save_mat(fnm + '_mc.mat', groups=['orient', 'main'])
        drm.pop_data('datenum')

        print("Averaging...")
        bdat = bnr(dr)
        ti = bdat.sigma_Uh / bdat.U_mag
        ti[bdat.U_mag < 0.7] = np.NaN

        print("Saving average...")
        np.savetxt(fnm + '_Average5min.csv',
                   np.vstack((num2date(bdat.mpltime), bdat.u, bdat.v, bdat.w, ti)).T,
                   fmt=['%s'] + ['%0.3f'] * 4,
                   header='Date+Time (US/Pacific),'
                   ' u (true east m/s), v (true north m/s), w (up m/s), Turbulence Intensity',
                   delimiter=', ')

        print("Done.")
